# ------------------------------------------------------------
# Fitting the whole dataset
# ------------------------------------------------------------

fit.all.train2test = function(ses, models = models.by.name)
   {ts = transform(
        ss(ts, s %in% ses & type %in% qw(test, train)),
        train = type == "train",
        test = type == "test")
    lapply.names(models, function(modelname)
       {message("~~~ ", modelname, " ~~~")
        builder.test.model.multisubjects(
            ss(ts, s %in% ss(sb, model == modelname)$s),
            models[[modelname]])})}

fit.all.test2test = function(ses, models = models.by.name)
   {ts = transform(
        ss(ts, s %in% ses & type == "test"),
        train = T,
        test = T)
    lapply.names(models, function(modelname)
       {message("~~~ ", modelname, " ~~~")
        builder.test.model.multisubjects(
            ss(ts, s %in% ss(sb, model == modelname)$s),
            models[[modelname]])})}

fit.all.holdout = function(ses, holdout.factor = 1/3, models = models.by.name, ts = ts)
   {ts = transform(ss(ts, s %in% ses & type == "test"))
    ts$train = T
    ts$test = F
    for (the.s in ses)
       {test.i = which(with(ts, s == the.s))
        test.i = sample(test.i, ceiling(holdout.factor * length(test.i)))
        ts[test.i, "train"] = F
        ts[test.i, "test"] = T}
    lapply.names(models, function(modelname)
       {message("~~~ ", modelname, " ~~~")
        builder.test.model.multisubjects(
            ts,
            models[[modelname]])})}

fit.all.xvalid = function(ses, folds = 5, models = models.by.name, ts)
   {ts = transform(ss(ts, s %in% ses & type == "test"))
    ts$fold = NA
    for (the.s in ses)
       {i = which(with(ts, s == the.s))
        ts[i, "fold"] = sample(rep(1 : folds, len = length(i)))}
    ts = ordf(ts, s, type, qn)
    lapply.names(models, function(modelname)
       {message("~~~ ", modelname, " ~~~")
        model = models[[modelname]]
        ddply(ts, .(s), .progress = "text", function(slice)
           {print(char(slice$s[1]))
            do.call(rbind, lapply(1 : folds, function(fold.n)
                {d = ss(summarize.posterior(
                        model$sample.posterior(ss(slice, fold != fold.n)),
                        ss(slice, fold == fold.n),
                        model$choice.p),
                    !is.na(trial))
                 d$actual = ss(slice, fold == fold.n)$choice
                 d}))})})}

# ------------------------------------------------------------
# Summarizing 'fitall' objects
# ------------------------------------------------------------

show.fit.all = function(fitall)
   {for (k in names(fitall))
       {message("---", k, "---");
        print(fitall[[k]]$each.s[,1:6])}
    for (k in names(fitall))
       {message("---", k, "---");
        print(round(digits = 3, as.matrix(
            fitall[[k]]$each.s[,c(-1, -2, -3, -4, -6)])))}}

show.all.predictions = function(fitall)
   {x = ordf(
        ss(do.call(rbind,
            lapply(names(fitall), function (m)
                cbind(fitall[[m]]$tmm, model = m))),
            !is.na(trial)),
        model, s, trial)
    x = transform(x,
        s = factor(s, levels =
            row.names(ordf(sb[levels(x$s),], test.lls))),
        pred = logi2factor(mean > .5, qw(ss, ll)))
    ggplot(data = x) +
        geom_tile(aes(
            x = s,
            y = trial,
            fill = factor(interaction(pred, actual),
                levels = qw(ll.ll, ss.ss, ll.ss, ss.ll)))) +
        scale_fill_manual(name = "pred.obs", values = c(
            ll.ll = "black",
            ss.ss = "white",
            ll.ss = "#aa0000",
            ss.ll = "#aa8800")) +
        no.gridlines() +
        scale_y_quartet +
        scale_x_discrete(breaks = NULL) +
        theme(
            axis.text.x = element_blank(),
            axis.text.y = element_text(size = 9, hjust = 1)) +
        facet_grid(. ~ model, scales = "free")}

boot.correctness.matrix = function(fitall, ...)
   boot.ranks(value, L1,
       melt(lapply(fitall, function(x)
           x$each.s$correct / 100)),
       greater = T,
       ...)

boot.rmse.matrix = function(fitall, ...)
   boot.ranks(value, L1,
       melt(lapply(fitall, function(x)
           daply(ss(x$tmm, !is.na(trial)), .(factor(s)), function(slice)
               sqrt(mean((slice$mean - (slice$actual == "ll"))^2))))),
       # There's no need to actually do the square root, since
       # the square root function is monotonic.
       ...)

boot.correctness = function(fitall, boot.n = 5000,
        f = function (x) x$each.s$correct / 100,
        agg = mean)
   {scores = lapply.names(fitall, function(m) f(fitall[[m]]))
    models = names(sort(dec = T, simplify2array(lapply.names(fitall,
        function(m) agg(scores[[m]])))))
    do.call(rbind, lapply(1 : (length(models) - 1), function (i)
       {m1 = models[[i]]
        m2 = models[[i + 1]]
        data.frame(
            m1 = m1,
            m1.mean = round(digits = 3, agg(f(fitall[[m1]]))),
            m2 = m2,
            m2.mean = round(digits = 3, agg(f(fitall[[m2]]))),
            confidence = round(digits = 2, mean(
                replicate(boot.n,
                    agg(sample(scores[[m1]], rep = T)) >  
                    agg(sample(scores[[m2]], rep = T))))))}))}

boot.rmse = function(fitall, boot.n = 5000)
    boot.correctness(fitall, boot.n,
        f = function(x) daply(ss(x$tmm, !is.na(trial)), .(factor(s)), function(slice)
            with(slice, sqrt(mean( (mean - (actual == "ll"))^2 )))))
